import re
import string
from datetime import datetime


# fio normalize
# -------------------------------------------------------------
def replacer(txt: str) -> str:
    """Замена латинских букв на русские, если такие встретятся в ФИО."""
    symbols = ("ankbtmxcepwvANKBTMXCEPWV",
               "анквтмхсерввАНКВТМХСЕРВВ")
    tr = {ord(a): ord(b) for a, b in zip(*symbols)}
    return txt.translate(tr)


def is_latin(fio: str) -> bool:
    """Проверка, является ли ФИО написанным латинскими буквами."""
    return all('a' <= char <= 'z' or 'A' <= char <= 'Z' or char in ['-', ' '] for char in fio)


def dash_clear(fio: str) -> str:
    """Удаление "-" из ФИО"""
    fio = " ".join([x.strip().lstrip("-").rstrip("-").strip() for x in fio.encode().decode().split()
                    if x.strip() and x.strip() != "-"])

    if fio.count("-") > 1:
        fio_norm = []
        for ch in fio.split():
            if ch.count("-") > 1:
                fio_norm.append("-".join([x.strip().lstrip("-").rstrip("-") for x in ch.split("-")
                                          if x.strip() and x.strip() != "-"]))
            else:
                fio_norm.append(ch)
        fio = " ".join(fio_norm)
    return fio


def fio_normalize(fio: str) -> str:
    """Нормализация ФИО. Проверка, является или ФИО написанным латиницей. Удаление цифр, тире."""
    fio = fio.strip().lower()
    fio = re.sub(r'[^a-zа-я-ё ]', "", fio).strip()
    fio = dash_clear(fio)
    if is_latin(fio):
        fio = fio.title()
    else:
        fio = "".join([replacer(x) if x in string.ascii_letters else x for x in fio])
        fio = re.sub(r'[^а-я-ё ]', '', fio).title()
    return fio if fio.strip() else ""


# phone normalize
# -------------------------------------------------------------
def phone_normalize(phone: str) -> str:
    """Нормализация номера телефона.
    Приведение номера к единому виду с семеркой в начале, если это сотовый телефон."""
    phone = re.sub(r'[^0-9]', '', phone.encode().decode())
    if phone.strip().isdigit():
        if len(phone.strip()) > 11:
            return ""
        elif 6 <= len(phone.strip()) < 10:
            return phone.strip()
        elif len(phone.strip()) == 10:
            if phone.strip().startswith("9"):
                return f"7{phone.strip()}"
            else:
                return phone.strip()
        elif len(phone.strip()) == 11:
            if phone.strip().startswith("8") and phone.strip()[1] == "9":
                return f"7{phone.strip()[1:]}"
            elif phone.strip().startswith("7"):
                return phone.strip()
            else:
                return ""
        return ""
    return ""


# email normalize
# -------------------------------------------------------------
def validate_email(mail: str) -> bool:
    """
    Простая валидация e-mail на наличие левой части до @, на наличие в правой части точки, а также
    на количество букв в домене верхнего уровня.
    """
    if "@" in mail:
        if mail.strip().split("@")[0].strip():
            if "." in mail.strip().split("@")[1]:
                if len(mail.strip().split("@")[1].strip().split(".")[1]) > 1:
                    return True
                return False
            return False
        return False
    return False


def email_normalize(mail: str) -> str:
    """Нормализация e-mail, с проверками наличия необходимых атрибутов и валидацией."""
    mail = mail.encode().decode()
    if "@" in mail:
        mail = mail.strip().split("@")
        if mail[0].strip():
            if "." in mail[1]:
                ch_mail = mail[1].split(".")
                if len(ch_mail) >= 2:
                    return "@".join(mail) if validate_email("@".join(mail)) else ""
            else:
                two_side = re.sub(r'[,:;!_*+()/#%&-\\]', '.', mail[1])
                return "@".join((mail[0], two_side)) if validate_email("@".join((mail[0], two_side))) else ""
        return ""
    return ""


# birthdate normalize
# -------------------------------------------------------------
def bdate_normalize(bdate: str) -> str:
    """Нормализация даты рождения, если это возможно. Иначе, возвращаем в том виде, как и получили"""
    bdate = bdate.strip().replace(" ", "")
    if bdate:
        if bdate.isdigit():
            bdate = datetime.fromtimestamp(int(bdate)).strftime('%d.%m.%Y')
            return bdate
        if "-" in bdate and bdate.count("-") == 2:
            bdate = bdate.split("-")
            if len(bdate[0]) == 4:
                bdate = f"{bdate[2]}.{bdate[1]}.{bdate[0]}"
                return bdate
            if len(bdate[2]) == 4:
                bdate = f"{bdate[0]}.{bdate[1]}.{bdate[2]}"
                return bdate
        elif "/" in bdate and bdate.count("/") == 2:
            bdate = bdate.split("/")
            if len(bdate[0]) == 4:
                bdate = f"{bdate[2]}.{bdate[1]}.{bdate[0]}"
                return bdate
            if len(bdate[2]) == 4:
                bdate = f"{bdate[0]}.{bdate[1]}.{bdate[2]}"
                return bdate
        else:
            return bdate
    else:
        return ""


# other field check/count field check
# -------------------------------------------------------------
def other_check(heads_list: list, heads: list, row: list) -> str:
    """Получение значений из столбцов, не являющихся основными."""
    other = []
    for item in heads:
        if item not in heads_list and row[heads.index(item)].strip():
            other.append(f"{item}: {row[heads.index(item)].replace("'", "").replace('"', '')}")
    return "---".join(other)


def count_val(items: list) -> bool:
    """Подсчет количества пустых ячеек в строке."""
    return True if sum(1 for x in items if x) > 1 else False
